from enum import Enum


class By(Enum):
    NAME_IN_ENGINE = 100
    PATH_IN_ENGINE = 101
    TEXT_NATIVE = 102
    CONDITIONS_NATIVE = 103
    IMAGE_TEMPLATE = 104
    ID_IN_UICONFIG = 105
    ELEMENT_IN_ENGINE = 106
    ELEMENT_NATIVE = 107